#!/usr/bin/ruby

require "yaml"
require "json"
require "trollop"
require "open3"
require "awesome_print"
require "colorize"
require "forwardable"
require "gclouder_undefined_resources/cli_args"
require "gclouder_undefined_resources/shell"
require "gclouder_undefined_resources/local_resources"
require "gclouder_undefined_resources/remote_resources"
require "gclouder_undefined_resources/remote_resources/remote"
require "gclouder_undefined_resources/remote_resources/resource"
require "gclouder_undefined_resources/remote_resources/resource/symbols"

module GClouderUndefinedResources
  def self.run
    CLIArgs.load

    authenticate(CLIArgs.cli_args[:key_file]) if CLIArgs.cli_args[:key_file_given]

    undefined
  end

  private

  def self.set_project_id
    Shell.run("gcloud config set core/project #{local_resources['project_id']}")
  end

  def self.authenticate(key_file)
    set_project_id
    Shell.run("gcloud --quiet auth activate-service-account --key-file #{key_file}")
  end

  def self.local_resources
    @local_resources ||= LocalResources.new(CLIArgs.cli_args[:local_resources])
  end

  def self.undefined
    remote_resources = RemoteResources.new(local_resources["project_id"])
    remote_resources.collect

    case CLIArgs.cli_args[:group_by].to_sym
    when :location
      remote_resources.display_by_location
    when :type
      remote_resources.display_by_type
    else
      puts "unknown group-by type"
    end

    remote_resources.display_report unless CLIArgs.cli_args[:hide_report]

    exit 1 if remote_resources.any_undefined_and_unfiltered?
  end
end
